from Acquisition import aq_inner
from zope import interface, component

from Products.CMFCore.utils import getToolByName
from Products.CMFCore import permissions
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.app.layout.viewlets.common import ViewletBase
from plone.memoize.instance import memoize

from raptus.article.core import RaptusArticleMessageFactory as _
from raptus.article.core import interfaces
from raptus.article.media.interfaces import IVideos as IVideoProvider, ITeaser, IVideoEmbed, IVideoEmbedder

class IVideos(interface.Interface):
    """ Marker interface for the videos viewlet
    """

class Component(object):
    """ Component which lists the video files
    """
    interface.implements(interfaces.IComponent, interfaces.IComponentSelection)
    component.adapts(interfaces.IArticle)
    
    title = _(u'Videos')
    description = _(u'List of the video files contained in the article.')
    image = '++resource++video.gif'
    interface = IVideos
    viewlet = 'raptus.article.media.video'
    
    def __init__(self, context):
        self.context = context

class Viewlet(ViewletBase):
    """ Viewlet listing the video files
    """
    index = ViewPageTemplateFile('video.pt')
    
    def _class(self, brain, i, l):
        cls = []
        if i == 0:
            cls.append('first')
        if i == l-1:
            cls.append('last')
        if i % 2 == 0:
            cls.append('odd')
        if i % 2 == 1:
            cls.append('even')
        return ' '.join(cls)

    @property
    @memoize
    def show_title(self):
        """hide or show title. this can be set with a property in portal_porperties. 
        """
        props = getToolByName(self.context, 'portal_properties').raptus_article
        return props.getProperty('media_video_title', True)

    @property
    @memoize
    def videos(self):
        provider = IVideoProvider(self.context)
        manageable = interfaces.IManageable(self.context)
        items = manageable.getList(provider.getVideos())
        i = 0
        l = len(items)
        for item in items:
            item.update({'title': item['brain'].Title,
                         'description': item['brain'].Description,
                         'class': self._class(item['brain'], i, l),
                         'url': item['brain'].getURL(),
                         'embed': False})
            if IVideoEmbed.providedBy(item['obj']):
                embedders = component.getAdapters((item['obj'],), IVideoEmbedder)
                item['embed'] = True
                item['url'] = item['brain'].getRemoteUrl
                for name, embedder in embedders:
                    if embedder.matches():
                        item['code'] = embedder.getEmbedCode()
                        item['provider'] = embedder.name
            else:
                img = ITeaser(item['obj'])
                item['img'] = img.getTeaser()
            i += 1
        return items



"""Definition of the Audio content type
"""
from zope.interface import implements

try:
    from Products.LinguaPlone import public as atapi
except ImportError:
    # No multilingual support
    from Products.Archetypes import atapi
from Products.ATContentTypes.content import schemata
from Products.ATContentTypes.content import file
from Products.validation import V_REQUIRED

from Products.ContentTypeValidator.validator import ContentTypeValidator

try:
    from plone.app.blob.field import BlobField as FileField
except ImportError:
    from Products.Archetypes.atapi import FileField

from raptus.article.media.interfaces import IAudio
from raptus.article.media.config import PROJECTNAME
from raptus.article.core import RaptusArticleMessageFactory as _

AudioSchema = file.ATFileSchema.copy() + atapi.Schema((
        FileField('file',
                required=True,
                primary=True,
                searchable=False,
                languageIndependent=False,
                storage = atapi.AnnotationStorage(migrate=True),
                validators = (('isNonEmptyFile', V_REQUIRED),
                              ('checkFileMaxSize', V_REQUIRED),
                              ContentTypeValidator(('audio/mpeg', 'audio/x-mp3', 'audio/x-mpeg', 'audio/mp3',))),
                widget = atapi.FileWidget(
                        description = '',
                        label=_(u'label_audio', default=u'Audio file'),
                        show_content_type = False,),
        ),
    ))

AudioSchema['title'].storage = atapi.AnnotationStorage()
AudioSchema['description'].storage = atapi.AnnotationStorage()

for field in ('creators','allowDiscussion','contributors','location','language', 'nextPreviousEnabled', 'rights' ):
    if AudioSchema.has_key(field):
        AudioSchema[field].widget.visible = {'edit': 'invisible', 'view': 'invisible'}

schemata.finalizeATCTSchema(AudioSchema, folderish=False, moveDiscussion=True)

class Audio(file.ATFile):
    """An audio file"""
    implements(IAudio)
    
    portal_type = "Audio"
    schema = AudioSchema

    title = atapi.ATFieldProperty('title')
    description = atapi.ATFieldProperty('description')

atapi.registerType(Audio, PROJECTNAME)



"""Definition of the Video content type
"""
from AccessControl import ClassSecurityInfo
from zope.interface import implements

try:
    from Products.LinguaPlone import public as atapi
except ImportError:
    # No multilingual support
    from Products.Archetypes import atapi
from Products.CMFCore.permissions import View
from Products.ATContentTypes.content import schemata
from Products.ATContentTypes.content import file
from Products.ATContentTypes.content.base import ATCTFileContent
from Products.ATContentTypes.configuration import zconf
from Products.validation import V_REQUIRED

from Products.ContentTypeValidator.validator import ContentTypeValidator

try:
    from plone.app.blob.field import BlobField as FileField, ImageField
except ImportError:
    from Products.Archetypes.atapi import FileField, ImageField

from raptus.article.media.interfaces import IVideo
from raptus.article.media.config import PROJECTNAME
from raptus.article.core import RaptusArticleMessageFactory as _

valid_types = ['video/x-flv', 'application/x-flash-video', 'flv-application/octet-stream', 'video/flv', 'video/mp4', 'video/mp4v-es', 'video/x-m4v']
try:
    from stxnext.transform.avi2flv import avi_to_flv
    convert_types = ('video/x-avi', 'video/x-msvideo', 'video/x-ms-wmv', 'video/quicktime',)
    def register_patch():
        return avi_to_flv.Avi2Flv(inputs=convert_types)
    avi_to_flv.register = register_patch
    valid_types.extend(convert_types)
except:
    pass

VideoSchema = file.ATFileSchema.copy() + atapi.Schema((
        FileField('file',
                required=True,
                primary=True,
                searchable=False,
                languageIndependent=False,
                storage = atapi.AnnotationStorage(migrate=True),
                validators = (('isNonEmptyFile', V_REQUIRED),
                              ('checkFileMaxSize', V_REQUIRED),
                              ContentTypeValidator(valid_types)),
                widget = atapi.FileWidget(
                        description = '',
                        label=_(u'label_video', default=u'Video file'),
                        show_content_type = False,),
        ),
        ImageField('image',
                required=False,
                languageIndependent=False,
                storage = atapi.AnnotationStorage(migrate=True),
                swallowResizeExceptions = zconf.swallowImageResizeExceptions.enable,
                pil_quality = zconf.pil_config.quality,
                pil_resize_algo = zconf.pil_config.resize_algo,
                max_size = zconf.ATImage.max_image_dimension,
                sizes= {'large'   : (768, 768),
                        'preview' : (400, 400),
                        'mini'    : (200, 200),
                        'thumb'   : (128, 128),
                        'tile'    :  (64, 64),
                        'icon'    :  (32, 32),
                        'listing' :  (16, 16),
                       },
                validators = (('isNonEmptyFile', V_REQUIRED),
                              ('checkImageMaxSize', V_REQUIRED)),
                widget = atapi.ImageWidget(
                        description = '',
                        label= _(u'label_image', default=u'Image'),
                        show_content_type = False,)
        ),
    ))

VideoSchema['title'].storage = atapi.AnnotationStorage()
VideoSchema['description'].storage = atapi.AnnotationStorage()

schemata.finalizeATCTSchema(VideoSchema, folderish=False, moveDiscussion=True)

class Video(file.ATFile):
    """A video file"""
    implements(IVideo)
    
    portal_type = "Video"
    schema = VideoSchema

    security = ClassSecurityInfo()

    title = atapi.ATFieldProperty('title')
    description = atapi.ATFieldProperty('description')

    security.declareProtected(View, 'tag')
    def tag(self, **kwargs):
        """Generate image tag using the api of the ImageField
        """
        return self.getField('image').tag(self, **kwargs)

    def __bobo_traverse__(self, REQUEST, name):
        """Transparent access to image scales
        """
        if name.startswith('image'):
            field = self.getField('image')
            image = None
            if name == 'image':
                image = field.getScale(self)
            else:
                scalename = name[len('image_'):]
                if scalename in field.getAvailableSizes(self):
                    image = field.getScale(self, scale=scalename)
            if image is not None and not isinstance(image, basestring):
                # image might be None or '' for empty images
                return image

        return ATCTFileContent.__bobo_traverse__(self, REQUEST, name)

atapi.registerType(Video, PROJECTNAME)



"""Common configuration constants
"""
from AccessControl import ModuleSecurityInfo
from Products.CMFCore.permissions import setDefaultRoles

PROJECTNAME = 'raptus.article.media'

security = ModuleSecurityInfo('raptus.article.media.config')

security.declarePublic('ADD_PERMISSION')
ADD_PERMISSION = {}
ADD_PERMISSION['Video'] = 'raptus.article: Add Video'
setDefaultRoles(ADD_PERMISSION['Video'], ('Manager','Contributor',))

ADD_PERMISSION['VideoEmbed'] = 'raptus.article: Add Embeded Video'
setDefaultRoles(ADD_PERMISSION['VideoEmbed'], ('Manager','Contributor',))

ADD_PERMISSION['Audio'] = 'raptus.article: Add Audio'
setDefaultRoles(ADD_PERMISSION['Audio'], ('Manager','Contributor',))



from zope import interface, component

from Products.CMFCore.utils import getToolByName

from raptus.article.core.interfaces import IArticle
from raptus.article.media.interfaces import IVideos, IAudios, IVideo, ITeaser

class Videos(object):
    """ Provider for video files contained in an article
    """
    interface.implements(IVideos)
    component.adapts(IArticle)
    
    def __init__(self, context):
        self.context = context
        
    def getVideos(self, **kwargs):
        """ Returns a list of video files (catalog brains)
        """
        catalog = getToolByName(self.context, 'portal_catalog')
        return catalog(portal_type=['Video', 'VideoEmbed',], path={'query': '/'.join(self.context.getPhysicalPath()),
                                                                   'depth': 1}, sort_on='getObjPositionInParent', **kwargs)

class Teaser(object): 
    """ Handler for image thumbing of videos
    """
    interface.implements(ITeaser)
    component.adapts(IVideo)
    
    def __init__(self, context):
        self.context = context
        
    def getTeaserURL(self, size='video'):
        """
        Returns the url to the teaser image in the specific size
        
        The sizes are taken from the raptus_article properties sheet
        and are formed by the following name schema:
        
            media_<size>_(height|width)
        """
        if not self.context.Schema()['image'].get(self.context):
            return None
        
        w, h = self.getSize(size)
        
        url = '%s/image' % self.context.absolute_url()
        if w or h:
            scales = component.getMultiAdapter((self.context, self.context.REQUEST), name='images')
            scale = scales.scale('image', width=(w or 100000), height=(h or 100000))
            if scale is not None:
                url = scale.url
        return url
        
    def getTeaser(self, size='video'):
        """ 
        Returns the html tag of the teaser image in the specific size
        
        The sizes are taken from the raptus_article properties sheet
        and are formed by the following name schema:
        
            media_<size>_(height|width)
        """
        url = self.getTeaserURL(size)
        if not url:
            return None
        
        return '<img src="%s" alt="" />' % url
        
    def getSize(self, size='video'):
        """
        Returns the width and height registered for the specific size
        """
        props = getToolByName(self.context, 'portal_properties').raptus_article
        return props.getProperty('media_%s_width' % size, 0), props.getProperty('media_%s_height' % size, 0)

class Audios(object):
    """ Provider for audio files contained in an article
    """
    interface.implements(IAudios)
    component.adapts(IArticle)
    
    def __init__(self, context):
        self.context = context
        
    def getAudios(self, **kwargs):
        """ Returns a list of audio files (catalog brains)
        """
        catalog = getToolByName(self.context, 'portal_catalog')
        return catalog(portal_type='Audio', path={'query': '/'.join(self.context.getPhysicalPath()),
                                                  'depth': 1}, sort_on='getObjPositionInParent', **kwargs)



from zope import component
from zope.interface.interfaces import IInterface
from collective.flowplayer.interfaces import IFlowPlayerSite

from Products.CMFCore.utils import getToolByName

def install(context):
    """ Removes the persistent utility registered by collective.flowplayer
    """
    if context.readDataFile('raptus.article.media_install.txt') is None:
        return
    portal = context.getSite()
    manager = component.getSiteManager()
    
    util = manager.queryUtility(IInterface, name='collective.flowplayer.interfaces.IFlowPlayerSite')
    manager.unregisterUtility(util, IInterface, name='collective.flowplayer.interfaces.IFlowPlayerSite')
    del util
    try: # try deleting subscriber if available
        del manager.utilities._subscribers[0][IInterface]
    except:
        pass
    
    # install stxnext.transform.avi2flv if available
    inst = getToolByName(portal, 'portal_quickinstaller')
    prod = 'stxnext.transform.avi2flv'
    try:
        if not inst.isProductInstalled(prod):
            inst.installProduct(prod)
        else:
            inst.reinstallProducts(prod)
    except:
        pass



"""Definition of the VideoEmbed content type
"""
from zope.interface import implements

try:
    from Products.LinguaPlone import public as atapi
except ImportError:
    # No multilingual support
    from Products.Archetypes import atapi
from Products.ATContentTypes.content import schemata
from Products.ATContentTypes.content import link

from raptus.article.media.interfaces import IVideoEmbed
from raptus.article.media.config import PROJECTNAME
from raptus.article.core import RaptusArticleMessageFactory as _

VideoEmbedSchema = link.ATLinkSchema.copy()

VideoEmbedSchema['title'].required = False
VideoEmbedSchema['title'].storage = atapi.AnnotationStorage()
VideoEmbedSchema['description'].storage = atapi.AnnotationStorage()
VideoEmbedSchema['remoteUrl'].storage = atapi.AnnotationStorage()

schemata.finalizeATCTSchema(VideoEmbedSchema, folderish=False, moveDiscussion=True)

class VideoEmbed(link.ATLink):
    """An embedded video"""
    implements(IVideoEmbed)
    
    portal_type = "VideoEmbed"
    schema = VideoEmbedSchema

    title = atapi.ATFieldProperty('title')
    description = atapi.ATFieldProperty('description')
    remoteUrl = atapi.ATFieldProperty('remoteUrl')

atapi.registerType(VideoEmbed, PROJECTNAME)



import re
from urlparse import urlparse
from cgi import parse_qs

from zope import interface, component

from Products.CMFCore.utils import getToolByName

from raptus.article.media import interfaces

class BaseEmbedder(object):
    interface.implements(interfaces.IVideoEmbedder)
    component.adapts(interfaces.IVideoEmbed)
    
    _template = """"""
    _expression = ""
    
    def __init__(self, context):
        self.context = context
        
    def matches(self):
        return re.match(self._expression, self.context.getRemoteUrl()) is not None
    
    def getEmbedCode(self):
        props = getToolByName(self.context, 'portal_properties').raptus_article
        return self._template % dict(url=self.getUrl(),
                                     width=props.getProperty('media_video_width', 0),
                                     height=props.getProperty('media_video_height', 0))
    
class YouTube(BaseEmbedder):
    name = "YouTube"
    _template = """
    <object width="%(width)s" height="%(height)s">
      <param name="wmode" value="transparent"></param>
      <param name="movie" value="%(url)s"></param>
      <param name="allowFullScreen" value="true"></param>
      <param name="allowscriptaccess" value="always"></param>
      <embed src="%(url)s" type="application/x-shockwave-flash" allowscriptaccess="always" allowfullscreen="true" wmode="transparent" width="%(width)s" height="%(height)s"></embed>
    </object>
    """
    _expression = "^http(s)?://(www\.)?youtube\.com"
    
    def getUrl(self):
        url = urlparse(self.context.getRemoteUrl())
        qs = parse_qs(url[4])
        return "http://www.youtube.com/v/%s" % qs.get('v', ['',])[0]
    
class GoogleVideo(BaseEmbedder):
    name = "GoogleVideo"
    _template = """
    <object width="%(width)s" height="%(height)s">
      <param name="wmode" value="transparent"></param>
      <param name="movie" value="%(url)s"></param>
      <param name="allowFullScreen" value="true"></param>
      <param name="allowscriptaccess" value="always"></param>
      <embed id="VideoPlayback" src="%(url)s" style="width:%(width)spx; height:%(height)spx;" allowFullScreen="true" allowScriptAccess="always" type="application/x-shockwave-flash"></embed>
    </object>
    """
    _expression = "^http(s)?://video\.google\.com"
    
    def getUrl(self):
        url = urlparse(self.context.getRemoteUrl())
        qs = parse_qs(url[4])
        return "http://video.google.com/googleplayer.swf?docid=%s" % qs.get('docid', ['',])[0]
    
class Vimeo(BaseEmbedder):
    name = "Vimeo"
    _template = """
    <object width="%(width)s" height="%(height)s"><param name="allowfullscreen" value="true" />
      <param name="allowscriptaccess" value="always" />
      <param name="movie" value="%(url)s" />
      <embed src="%(url)s" type="application/x-shockwave-flash" allowfullscreen="true" allowscriptaccess="always" width="%(width)s" height="%(height)s"></embed>
    </object>
    """
    _expression = "^http(s)?://(www\.)?vimeo\.com"
    
    def getUrl(self):
        url = urlparse(self.context.getRemoteUrl())
        return "http://vimeo.com/moogaloop.swf?clip_id=%s&amp;server=vimeo.com" % url[2].split('/')[1]
    


from Acquisition import aq_inner
from zope import interface, component

from Products.CMFCore.utils import getToolByName
from Products.CMFCore import permissions
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.app.layout.viewlets.common import ViewletBase
from plone.memoize.instance import memoize

from raptus.article.core import RaptusArticleMessageFactory as _
from raptus.article.core import interfaces
from raptus.article.media.interfaces import IAudios as IAudioProvider

class IAudios(interface.Interface):
    """ Marker interface for the audios viewlet
    """

class Component(object):
    """ Component which lists the audio files
    """
    interface.implements(interfaces.IComponent, interfaces.IComponentSelection)
    component.adapts(interfaces.IArticle)
    
    title = _(u'Audio')
    description = _(u'List of the audio files contained in the article.')
    image = '++resource++audio.gif'
    interface = IAudios
    viewlet = 'raptus.article.media.audio'
    
    def __init__(self, context):
        self.context = context

class Viewlet(ViewletBase):
    """ Viewlet listing the audio files
    """
    index = ViewPageTemplateFile('audio.pt')
    
    def _class(self, brain, i, l):
        cls = []
        if i == 0:
            cls.append('first')
        if i == l-1:
            cls.append('last')
        if i % 2 == 0:
            cls.append('odd')
        if i % 2 == 1:
            cls.append('even')
        return ' '.join(cls)
    
    @property
    @memoize
    def audios(self):
        provider = IAudioProvider(self.context)
        manageable = interfaces.IManageable(self.context)
        items = manageable.getList(provider.getAudios())
        i = 0
        l = len(items)
        for item in items:
            item.update({'title': item['brain'].Title,
                         'description': item['brain'].Description,
                         'class': self._class(item['brain'], i, l),
                         'url': item['brain'].getURL()})
            i += 1
        return items



from zope import interface

class IVideos(interface.Interface):
    """ Provider for video files and embedded ones contained in an article
    """
    
    def getVideos(**kwargs):
        """ Returns a list of video files and embedded ones (catalog brains)
        """

class ITeaser(interface.Interface):
    """ Handler for image thumbing of videos
    """
        
    def getTeaserURL(size="original"):
        """
        Returns the url to the teaser image in the specific size
        
        The sizes are taken from the raptus_article properties sheet
        and are formed by the following name schema:
        
            media_<size>_(height|width)
        """
    
    def getTeaser(size="orginal"):
        """ 
        Returns the html tag of the teaser image in the specific size
        
        The sizes are taken from the raptus_article properties sheet
        and are formed by the following name schema:
        
            media_<size>_(height|width)
        """
        
    def getSize(size):
        """
        Returns the width and height registered for the specific size
        """

class IVideo(interface.Interface):
    """ Marker interface for the video content type
    """
        
class IVideoEmbed(interface.Interface):
    """ Marker interface for the video embed content type
    """
    
class IVideoEmbedder(interface.Interface):
    """ Provides html for different video providers
    """
    
    name = interface.Attribute('name', 'User friendly name of this embeder')
    
    def matches():
        """Whether the embeder matches the adapted obj or not
        """
    
    def getEmbedCode():
        """ Returns the html
        """

class IAudios(interface.Interface):
    """ Provider for audio files contained in an article
    """
    
    def getAudios(**kwargs):
        """ Returns a list of audio files (catalog brains)
        """

class IAudio(interface.Interface):
    """ Marker interface for the audio content type
    """



"""Main product initializer
"""
from raptus.article.media import config

from Products.Archetypes import atapi
from Products.CMFCore import utils as cmfutils

def initialize(context):
    from content import video, videoembed, audio

    content_types, constructors, ftis = atapi.process_types(
        atapi.listTypes(config.PROJECTNAME),
        config.PROJECTNAME)

    for atype, constructor in zip(content_types, constructors):
        cmfutils.ContentInit("%s: %s" % (config.PROJECTNAME, atype.portal_type),
            content_types      = (atype,),
            permission         = config.ADD_PERMISSION[atype.portal_type],
            extra_constructors = (constructor,),
            ).initialize(context)




import os
from Acquisition import aq_inner
from zope.annotation.interfaces import IAnnotations

from plone.app.blob.migrations import haveContentMigrations
from plone.app.blob.browser.migration import ImageMigrationView
from plone.app.blob.browser.migration import __file__ as location
from plone.app.blob.migrations import ATImageToBlobImageMigrator
from plone.app.blob.migrations import migrate
from plone.app.blob.migrations import getMigrationWalker

from Products.statusmessages.interfaces import IStatusMessage
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _


class ArticleMediaToBlobImageMigrator(ATImageToBlobImageMigrator):
    
    src_portal_type = 'Video'
    src_meta_type = 'Video'
    dst_portal_type = 'Video'
    dst_meta_type = 'Video'

    fields = ['file', 'image']
    
    def migrate_data(self):
        for field in self.fields:
            annotations = IAnnotations(self.old)
            res = annotations.get('Archetypes.storage.AnnotationStorage-%s'%field, None)
            if res:
                if field == 'file':
                    field = self.new.Schema()[field]
                    field.getMutator(self.new)(res)
                else:
                    self.new.Schema()[field].set(self.new, res)
                    del annotations['Archetypes.storage.AnnotationStorage-%s'%field]

    def createNew(self):
        self.new = self.old
        
    def remove(self):
        pass

    def reorder(self):
        pass
    
    def renameOld(self):
        pass

class ArticleAudioToBlobMigrator(ArticleMediaToBlobImageMigrator):

    src_portal_type = 'Audio'
    src_meta_type = 'Audio'
    dst_portal_type = 'Audio'
    dst_meta_type = 'Audio'
    fields = ['file']


class ArticleMediaMigrationView(ImageMigrationView):
    
    index = ViewPageTemplateFile('%s/migration.pt' % os.path.dirname(location))
    
    def __call__(self):
        context = aq_inner(self.context)
        request = aq_inner(self.request)
        walkers = self.walkers()
        target_type = '/'.join([i.src_portal_type for i in walkers])
        options = dict(target_type=target_type)
        clicked = request.form.has_key
        portal_url = getToolByName(context, 'portal_url')()
        if not haveContentMigrations:
            msg = _(u'Please install contentmigrations to be able to migrate to blobs.')
            IStatusMessage(request).addStatusMessage(msg, type='warning')
            options['nomigrations'] = 42
        elif clicked('migrate'):
            output = self.migrations()
            # Only count actual migration lines
            lines = output.split('\n')
            count = len([l for l in lines if l.startswith('Migrating')])
            msg = _(u'blob_migration_info',
                default=u'Blob migration performed for ${count} item(s).',
                mapping={'count': count})
            IStatusMessage(request).addStatusMessage(msg, type='info')
            options['count'] = count
            options['output'] = output
        elif clicked('cancel'):
            msg = _(u'Blob migration cancelled.')
            IStatusMessage(request).addStatusMessage(msg, type='info')
            request.RESPONSE.redirect(portal_url)
        else:
            li = list()
            for walker in walkers:
                li.extend(list( walker.walk()))
            options['available'] = len(li)
        return self.index(**options)
    
    def migrations(self):
        out = ''
        for walker in self.walkers():
            out += migrate(self, walker=lambda x: walker)
        return out
    
    def walkers(self):
        return [getMigrationWalker(self, migrator=ArticleMediaToBlobImageMigrator),
                getMigrationWalker(self, migrator=ArticleAudioToBlobMigrator) ]
    
    
    
    